#-----
#Sasssa Helper
#-----
from vcCommand import *
import math
import vcMatrix, vcVector

R2D = 180.0 / math.pi
D2R = math.pi / 180.0
EPSILON = 1e-6

app = getApplication()
sim = getSimulation()
cmd = getCommand()
snap = app.findCommand('interactiveSnap') 

#-----
def OnStart():
  global prop_plane, prop_driver, prop_select
  
  prop_plane = cmd.getProperty('Plane')
  prop_select = cmd.getProperty('Select Points')
  if not prop_plane:
    prop_plane = cmd.createProperty(VC_STRING, 'Plane', VC_PROPERTY_STEP)
    prop_plane.StepValues = ['XY', 'XZ', 'YZ', 'From selection']
    prop_plane.Value = 'From selection'
    prop_select = cmd.createProperty(VC_BUTTON, 'Select Points')
  
  prop_select.OnChanged = execute
  
  print '-----'
  print 'Functions'
  print '  a = sasad(A, b, C)'
  print '  B = sassd(A, b, C)'
  print '  a = sssad(A, B, C)'
  print '-----'
  
  executeInActionPanel()
#-----
def execute(*args):
  startSnap(3, ['Point a', 'Point b', 'Point c']) 
#-----
def endSnap():
  global targets
  global target_count, target_names
  
  im = vcMatrix.new(targets[0])
  im.invert()
  for i in range(len(targets)):
    if prop_plane.Value != 'From selection':
      m = targets[i]
      p = m.P
      if prop_plane.Value == 'XY':
        p.Z = 0
      elif prop_plane.Value == 'XZ':
        p.Y = 0
      else:#elif prop_plane.Value == 'YZ':
        p.X = 0
      m.P = p
    else:
      #Convert points to plane from first selection
      m = im * targets[i]
      p = m.P
      p.Z = 0
      m.P = p
      targets[i] = m
  
  pa = targets[0].P
  pb = targets[1].P
  pc = targets[2].P
  
  sab = pb - pa
  sbc = pc - pb
  sca = pa - pc
  
  A = sbc.length()
  B = sca.length()
  C = sab.length()
  
  a = 180 - sca.angle(sab) * R2D
  b = 180 - sab.angle(sbc) * R2D
  c = 180 - sbc.angle(sca) * R2D
  
  print '-----'
  print 'Functions (prefilled)'
  print '  a = sasad(A, b, C) = sasad(%.3f, %.3f, %.3f) = %.3f' % (A, b, C, a)
  print '  B = sassd(A, b, C) = sassd(%.3f, %.3f, %.3f) = %.3f' % (A, b, C, B)
  print '  a = sssad(A, B, C) = sssad(%.3f, %.3f, %.3f) = %.3f' % (A, B, C, a)
  print '-----'
  print 'Side lengths'
  print '  A: %.3f' % A
  print '  B: %.3f' % B
  print '  C: %.3f' % C
  print 'Corner angles'
  print '  a: %.3f' % a
  print '  b: %.3f' % b
  print '  c: %.3f' % c
  print '-----'
  
#-----
def updateSnap(snap):
  global targets
  global target_count, target_names
  
  #CODE here
  pass
#-----
def startSnap(count = 1, names = []):
  global targets
  global target_count, target_names
  
  targets = []
  target_count = count
  target_names = names
  snap.clear()
  snap.Snap = True 
  snap.SnapOnCircleCenter = True 
  snap.SnapOnCurve = True 
  snap.SnapOnEdge = True 
  snap.SnapOnEdgeCenter = True 
  snap.SnapOnFaceCenter = True 
  snap.SnapOnFrame = True 
  snap.SnapOnNodeCenter = False 
  snap.SnapOnNodeOrigin = False 
  snap.SnapOnPoint = True 
  snap.SnapOnPolygon = True 
  snap.SnapOnSurface = True 
  callSnap()
#-----
def callSnap():
  global targets
  global target_count, target_names
  
  snap.OnTargetAvailable = updateSnap
  snap.OnTargetSet = doSnap
  name = 'Target ' + str(len(targets) + 1)
  if len(target_names) > len(targets):
    name = target_names[len(targets)]
  print 'Select ' + name
  snap.executeAsTool()
#-----
def doSnap(snap):
  global targets
  global target_count, target_names
  
  snap.OnTargetAvailable = None
  snap.OnTargetSet = None
  if snap.TargetMatrixAvailable:
    targets.append(snap.TargetMatrix)
  else:
    target = vcMatrix.new()
    target.P = snap.TargetPosition
    targets.append(target)
  if len(targets) < target_count:
    callSnap()
  else:
    endSnap()
#-----