#-----
#Sasssa Helper
#-----
from vcCommand import *
import math
import vcMatrix, vcVector

R2D = 180.0 / math.pi
D2R = math.pi / 180.0
EPSILON = 1e-6

app = getApplication()
sim = getSimulation()
cmd = getCommand()
snap = app.findCommand('interactiveSnap') 

#-----
def OnStart():
  global prop_plane, prop_driver, prop_select
  
  prop_plane = cmd.getProperty('Plane')
  prop_select = cmd.getProperty('Select Points')
  if not prop_plane:
    prop_plane = cmd.createProperty(VC_STRING, 'Plane', VC_PROPERTY_STEP)
    prop_plane.StepValues = ['XY', 'XZ', 'YZ', 'From selection']
    prop_plane.Value = 'From selection'
    prop_select = cmd.createProperty(VC_BUTTON, 'Select Points')
  
  prop_select.OnChanged = execute
  
  print '-----'
  print 'Functions'
  print '  c = sasasad(A, a, B, b, C)'
  print '  D = sasassd(A, a, B, b, C)'
  print '  b = sasssa1d(A, a, B, C, D)'
  print '  c = sasssa2d(A, a, B, C, D)'
  print '-----'
  
  executeInActionPanel()
#-----
def execute(*args):
  startSnap(4, ['Point a', 'Point b', 'Point c', 'Point d']) 
#-----
def endSnap():
  global targets
  global target_count, target_names
  
  im = vcMatrix.new(targets[0])
  im.invert()
  for i in range(len(targets)):
    if prop_plane.Value != 'From selection':
      m = targets[i]
      p = m.P
      if prop_plane.Value == 'XY':
        p.Z = 0
      elif prop_plane.Value == 'XZ':
        p.Y = 0
      else:#elif prop_plane.Value == 'YZ':
        p.X = 0
      m.P = p
    else:
      #Convert points to plane from first selection
      m = im * targets[i]
      p = m.P
      p.Z = 0
      m.P = p
      targets[i] = m
  
  pa = targets[0].P
  pb = targets[1].P
  pc = targets[2].P
  pd = targets[3].P
  
  sab = pb - pa
  sbc = pc - pb
  scd = pd - pc
  sda = pa - pd
  
  A = sda.length()
  B = sab.length()
  C = sbc.length()
  D = scd.length()
  
  a = 180 - sda.angle(sab) * R2D
  b = 180 - sab.angle(sbc) * R2D
  c = 180 - sbc.angle(scd) * R2D
  d = 180 - scd.angle(sda) * R2D
  
  print '-----'
  print 'Functions (prefilled)'
  print '  c = sasasad(A, a, B, b, C)  = sasasad(%.3f, %.3f, %.3f, %.3f, %.3f)  = %.3f' % (A, a, B, b, C, c)
  print '  D = sasassd(A, a, B, b, C)  = sasassd(%.3f, %.3f, %.3f, %.3f, %.3f)  = %.3f' % (A, a, B, b, C, D)
  print '  b = sasssa1d(A, a, B, C, D) = sasssa1d(%.3f, %.3f, %.3f, %.3f, %.3f) = %.3f' % (A, a, B, C, D, b)
  print '  c = sasssa2d(A, a, B, C, D) = sasssa2d(%.3f, %.3f, %.3f, %.3f, %.3f) = %.3f' % (A, a, B, C, D, c)
  print '-----'
  print 'Side lengths'
  print '  A: %.3f' % A
  print '  B: %.3f' % B
  print '  C: %.3f' % C
  print '  D: %.3f' % D
  print 'Corner angles'
  print '  a: %.3f' % a
  print '  b: %.3f' % b
  print '  c: %.3f' % c
  print '  d: %.3f' % d
  print '-----'
  
#-----
def updateSnap(snap):
  global targets
  global target_count, target_names
  
  #CODE here
  pass
#-----
def startSnap(count = 1, names = []):
  global targets
  global target_count, target_names
  
  targets = []
  target_count = count
  target_names = names
  snap.clear()
  snap.Snap = True 
  snap.SnapOnCircleCenter = True 
  snap.SnapOnCurve = True 
  snap.SnapOnEdge = True 
  snap.SnapOnEdgeCenter = True 
  snap.SnapOnFaceCenter = True 
  snap.SnapOnFrame = True 
  snap.SnapOnNodeCenter = False 
  snap.SnapOnNodeOrigin = False 
  snap.SnapOnPoint = True 
  snap.SnapOnPolygon = True 
  snap.SnapOnSurface = True 
  callSnap()
#-----
def callSnap():
  global targets
  global target_count, target_names
  
  snap.OnTargetAvailable = updateSnap
  snap.OnTargetSet = doSnap
  name = 'Target ' + str(len(targets) + 1)
  if len(target_names) > len(targets):
    name = target_names[len(targets)]
  print 'Select ' + name
  snap.executeAsTool()
#-----
def doSnap(snap):
  global targets
  global target_count, target_names
  
  snap.OnTargetAvailable = None
  snap.OnTargetSet = None
  if snap.TargetMatrixAvailable:
    targets.append(snap.TargetMatrix)
  else:
    target = vcMatrix.new()
    target.P = snap.TargetPosition
    targets.append(target)
  if len(targets) < target_count:
    callSnap()
  else:
    endSnap()
#-----